export interface IAppConfig
{
	Components?: object;
	Development?: {
		enabled?: boolean;
		bodyClassName?: string;
		disablePrivateRedirects?: boolean;
	};
	User?: {
		useLogin?: boolean;
		defaultView?: string;
	};
	View?: {
		privateRedirectView?: string;
		defaultTheme?: string;
	};
	Database?: {
		userNameFieldName?: string;
		storageNameFieldName?: string;
		userIdFieldName?: string;
		storageIdFieldName?: string;
	};
	Custom?: object;
}

export interface IRoutesConfig
{
	routes: IRoutesConfigRoute[];
}

export interface IRoutesConfigRoute
{
	moduleId: string;
	name: string;
	route: string;
	title?: string;
	nav?: number;
}

export interface IRedirectDecoratorFunction extends Function
{
	onLoggedIn: Function;
	onLoggedOut: Function;
}

export interface IAppViewOptions
{
	superView: HTMLElement;
	routerView: HTMLElement;
	snackbar: any;
}

export interface IOnlyIfDecoratorFunction extends Function
{
	hasParameters: Function;
	hasProperties: Function;
	isOnline: Function;
	underDevelopment: Function;
}

export interface IValidatable
{
	valid: boolean;
	validate: () => boolean;
}

export interface IScrollMarkerHandlerConfig
{
	element: HTMLElement;
	className: string;
	minHeight?: number;
	maxHeight?: number;
}

type ViewFunction = ( config: IViewDecoratorConfig ) => ( constructor: any ) => any;

export interface IViewDecorator extends ViewFunction
{
	redirectTo: RedirectDecoratorFunction;
	public: SimpleDecoratorFunction;
	theme: SimpleDecoratorFunction;
}

export interface IViewDecoratorConfig
{
	redirectTo?: RedirectDecoratorFunction;
	public?: boolean;
	theme?: string;
}

export declare type ColorCallback = () => string;

export interface IPitayaViewSettings
{
	isPublic: boolean;
	hierarchy: number;
	theme: string;
}

export interface IViewStates
{
	active: boolean;
	created: boolean;
	bound: boolean;
	attached: boolean;
	animated: boolean;
	detached: boolean;
}

type AppViewFunction = ( config: IViewDecoratorConfig ) => ( constructor: any ) => any;

export interface IAppViewDecorator extends AppViewFunction
{
	title?: Function;
	routesFile?: Function;
	settingsFile?: Function;
	configure?: ( options: string | IAppConfig ) => any;
	routes?: ( routes: string | IRoutesConfigRoute[] ) => any;
}

export interface IAppViewDecoratorConfig
{
	topbar?: { actions?: string[]; };
	title?: string;
	routesFile?: string;
	settingsFile?: string;
}

type SimpleDecoratorFunction = ( constructor: any ) => any;
type RedirectDecoratorFunction = ( route: string, callback: ( instance, route ) => boolean ) => ( constructor: any ) => any;

export interface IRouteTransition extends String{}

export interface IDataFlowConnector
{
	name?: string;

	initialize(): Promise<any>;

	destroy(): Promise<any>;
}